@@ -1,7 +1,7 @@
 /* glbl.c: global command routines for the ed line editor */
 /*  GNU ed - The GNU line editor.
     Copyright (C) 1993, 1994 Andrew Moore, Talke Studio
-    Copyright (C) 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ static int active_ndx = 0;	/* active_list index ( modulo active_last ) */
 char set_active_node( const line_t *lp )
   {
   disable_interrupts();
-  if( !resize_buffer( (char **)(void *)&active_list, &active_size,
+  if( !resize_buffer( (void *)&active_list, &active_size,
                       ( active_last + 1 ) * sizeof( line_t ** ) ) )
     {
     show_strerror( 0, errno ); set_error_msg( "Memory exhausted" );
